<?php

declare(strict_types = 1);

namespace Drupal\entity_link_template_condition\Plugin\Condition;

use Drupal\Core\Condition\ConditionPluginBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides condition for filtering on link templates.
 *
 * @Condition(
 *   id = "entity_link_template",
 *   label = @Translation("Entity Link Template"),
 *   context_definitions = {
 *     "entity" = @ContextDefinition("entity", label = @Translation("Entity"))
 *   }
 * )
 */
class EntityLinkTemplateCondition extends ConditionPluginBase implements ContainerFactoryPluginInterface {

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Constructs a EntityLinkTemplateCondition condition plugin.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $routeMatch
   *   Current route match.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, RouteMatchInterface $routeMatch) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
    $this->routeMatch = $routeMatch;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('current_route_match'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'link_templates' => [],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $linkTemplates = $this->getLinkTemplates();
    sort($linkTemplates);
    $form['link_templates'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('When the user has the following roles'),
      '#default_value' => $this->configuration['link_templates'],
      '#options' => array_combine($linkTemplates, array_map(function (string $linkTemplate) {
        return $this->t('@link_template', ['@link_template' => $linkTemplate]);
      }, $linkTemplates)),
    ];
    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['link_templates'] = array_filter($form_state->getValue('link_templates'));
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function summary() {
    return $this->t('Link templates: @link_templates', [
      '@link_templates' => $this->configuration['link_templates'],
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function evaluate() {
    $entity = $this->getContextValue('entity');
    assert($entity instanceof EntityInterface);

    $entityLinkTemplates = $entity->getEntityType()->getLinkTemplates();
    $entityLinkTemplatesMatches = array_intersect_key($entityLinkTemplates, array_flip($this->configuration['link_templates']));

    $currentPath = $this->routeMatch->getRouteObject()->getPath();
    return in_array($currentPath, $entityLinkTemplatesMatches, TRUE);
  }

  /**
   * Get all link templates.
   *
   * @return string[]
   *   Link templates.
   */
  public function getLinkTemplates(): array {
    $linkTemplates = [];
    foreach ($this->entityTypeManager->getDefinitions() as $entityType) {
      array_push($linkTemplates, ...array_keys($entityType->getLinkTemplates()));
    }
    return array_unique($linkTemplates);
  }

}
